@@ -29,10 +29,13 @@ import transport
 import util
 import time
 import process as processing
+import logging
 
 __all__ = ('Message', 'ActorManager', 'Actor', 'PacketError', 'PacketTypeError', 'GroupAlreadyExists', 'GroupDoesNotExist', 'CreateGroupError',
            'DefaultActorFailed', 'GroupFailed', 'get_logger', 'WrongMessageTypeSpecified')
 
+GROUP_WARNING_MESSAGE = '''Please call mgr.enable_groups() first before using "groups" mode.  This ensures that your app is safe when "frozen" into an executable in Windows.  Also ensure any "add_process_group" calls happen under the main function (i.e.: if __name__ == '__main__' ...).  This is required under Windows.  See "Grouping" documentation.'''
+
 class PacketError(Exception):
     pass
 
@@ -60,28 +63,37 @@ class WrongMessageTypeSpecified(Exception):
 class ConcreteMessageAlreadyDefined(Exception):
     pass
 
+class GroupsNotEnabled(Exception):
+    pass
+
 def get_logger():
     return processing.get_logger()
     
 def _subprocess_main(name, default_actor_class, max_tick_time, interval, server_addr, _should_quit, packet_types):
     '''interval is in milliseconds of how long to sleep before another tick'''
     # creating a client mode manager
+    # after forking, we would already have an instance tied to the parent PID, simply change that to our PID
+    ActorManager._switch_instance_after_fork()
     manager = ActorManager.get_singleton()
+    manager.reset_to_client_mode()
+    # under non-forking systems, this would simply create a new manager
     # the new manager may not have all packet types registered, register them here
+    # on windows, packet types will be auto-registered
+    # on *nix, we would have whatever packets that were registered by the parent process
     if manager.packet_types:
-        # on windows, packet types will be auto-registered
         assert set(manager.packet_types) == set(packet_types)
     else:
-        # on *nix, forking would cause packets NOT be auto-registered
         manager.packet_types = packet_types
     manager._ipc_connect(server_addr, _should_quit)
-    processing.get_logger().info('process "%s" is bound to address: "%s"' % (processing.get_pid(processing.current_process()), manager.ipc_transport._connection.fileno()))
+    manager.log(logging.INFO, 'current process "%s" is bound to address: "%s"' % (processing.get_pid(processing.current_process()), manager.ipc_transport._connection.fileno()))
     try:
         default_actor = default_actor_class()
     except Exception, e:
         raise DefaultActorFailed('Default actor class "%s" failed to initialize. ("%s")' % (default_actor_class, e))
     else:
         manager.register_actor(default_actor)
+    # manager is now seen as a child
+    assert not manager.is_main_process
     while not manager._should_quit.value:
         start = util.get_time()
         manager.tick(max_time=max_tick_time)
@@ -106,7 +118,8 @@ class ActorManager(messaging.MessageManager):
         # using either Domain Socket (Unix) or Named Pipe (windows) as means
         # for IPC
         self.groups = {}
-        self.is_main_process = None
+        self.is_main_process = True
+        self._groups_enabled = False
         self.ipc_transport = transport.IPCTransport()
     def find(self, name):
         '''returns an actor by its name, None if not found'''
@@ -232,7 +245,7 @@ class ActorManager(messaging.MessageManager):
                 if cut_off_time and util.get_time() > cut_off_time:
                     break
 
-        processing.get_logger().debug('process "%s" queue length: %s' % (processing.get_pid(processing.current_process()), self.queue_length))
+        self.log(logging.DEBUG, 'process "%s" queue length: %s' % (processing.get_pid(processing.current_process()), self.queue_length))
         
         # process all messages first
         new_max_time = None
@@ -243,13 +256,14 @@ class ActorManager(messaging.MessageManager):
         # then update all actors
         map(lambda x: x.update(*args, **kws), sorted(self.objectIDMap.values(), lambda x,y: y._SYNC_PRIORITY - x._SYNC_PRIORITY))
         return ret
+    def log(self, level, msg):
+        '''process aware logging'''
+        return processing.get_logger().log(level, msg)
     def _ipc_listen(self):
         # starting server mode
-        self.is_main_process = True
         self.ipc_transport.listen()
     def _ipc_connect(self, server_addr, _should_quit):
         # starting client mode
-        self.is_main_process = False
         self.ipc_transport.connect(server_addr)
         self._should_quit = _should_quit
         self.groups[self.PYSAGE_MAIN_GROUP] = (None,server_addr,None)
@@ -263,7 +277,7 @@ class ActorManager(messaging.MessageManager):
             ` `transport_class`: optional.  the transport class that will be used to define the protocol
         '''
         def connection_handler(client_address):
-            processing.get_logger().debug('connected to client: %s' % client_address)
+            self.log(logging.DEBUG, 'connected to client: %s' % client_address)
         self.transport = transport_class()
         self.transport.listen(host, port, connection_handler)
         return self
@@ -278,7 +292,7 @@ class ActorManager(messaging.MessageManager):
         self.transport = transport_class()
         self.transport.connect(host, port)
         return self
-    def send_message(self, msg, address):
+    def send_message(self, msg, address=None):
         '''
         send a message to a network
         
@@ -293,19 +307,20 @@ class ActorManager(messaging.MessageManager):
         if not self.groups.has_key(group):
             raise GroupDoesNotExist('Group "%s" does not exist' % group)
         p, _clientid, switch = self.groups[group]
-        processing.get_logger().info('queuing message "%s" to "%s"' % (msg, _clientid))
+        self.log(logging.INFO, 'queuing message "%s" to "%s"' % (msg, _clientid))
         self.ipc_transport.send(msg.to_string(), _clientid)
     def broadcast_message(self, msg):
         self.transport.send(msg.to_string(), broadcast=True)
         return self
-    def packet_handler(self, packet):
-        packetid = ord(packet.data[0])
+    def packet_handler(self, packet, address):
+        packetid = ord(packet[0])
         processing.get_logger().debug('Received packet of type "%s"' % type)
         if packetid < 100:
             processing.get_logger().warning('internal packet unhandled: "%s"' % self.transport.packet_type_info(packetid))
             return self
-        p = self.packet_types[packetid]().from_string(packet.data)
-        self.queue_message(self.packet_types[packetid]().from_string(packet.data)) 
+        p = self.packet_types[packetid]().from_string(packet)
+        p.sender = address
+        self.queue_message(p)
         return self
     def register_packet_type(self, packet_class):
         # skip the base packet class
@@ -319,10 +334,18 @@ class ActorManager(messaging.MessageManager):
             raise PacketTypeError('Packet_type is already registered with packet "%s"' % self.packet_types[packet_class.packet_type])
         self.packet_types[packet_class.packet_type] = packet_class
         self.message_map[packet_class.__name__] = packet_class
+    def validate_groups_mode(self):
+        if not self._groups_enabled:
+            raise GroupsNotEnabled(GROUP_WARNING_MESSAGE)
+    def enable_groups(self):
+        '''enable freeze support'''
+        processing.enable_groups()
+        self._groups_enabled = True
     def add_process_group(self, name, default_actor_class=None, max_tick_time=None, interval=.03):
         '''adds a process group to the pool'''
-        if self.is_main_process == None:
-            self._ipc_listen()
+        assert self.is_main_process, 'Pysage currently only supports spawning child groups from the Main Group'
+        self.validate_groups_mode()
+        self._ipc_listen()
         # make sure we have a str
         g = str(name)
         if self.groups.has_key(g):
@@ -356,6 +379,15 @@ class ActorManager(messaging.MessageManager):
     @property
     def queue_length(self):
         return len(self.active_queue)
+    def reset_to_client_mode(self):
+        '''after forking in *nix systems, we need to clean up the current manager'''
+        super(ActorManager, self).reset_to_client_mode()
+        self.is_main_process = False
+        self.groups = {}
+        self.ipc_transport = transport.IPCTransport()
+        self.objectIDMap = {}
+        self.objectNameMap = {}
+        self.transport = None
     def reset(self):
         '''mainly used for testing'''
         messaging.MessageManager.reset(self)
@@ -371,7 +403,6 @@ class ActorManager(messaging.MessageManager):
         # not removing the auto-registered packet types
         # self.packet_types = {}
         self.groups = {}
-        self.is_main_process = None
         self.ipc_transport = transport.IPCTransport()
                 
 class Actor(messaging.MessageReceiver):
@@ -402,14 +433,14 @@ class Message(messaging.Message):
     def to_string(self):
         '''packs message into binary stream'''
         if not len(self.types) == len(self.properties):
-            raise WrongMessageTypeSpecified('Message "%s" has %s properties, but %s types' % (self, len(self.properties), len(self.types)))
+            raise WrongMessageTypeSpecified('Message "%s" has %s properties, but %s types specified.  Check the "types" and "properties" class attribute of the "%s" class' % (self, len(self.properties), len(self.types), type(self)))
         # first encode the message type identifier
         buf = struct.pack('!B', self.packet_type)
         # iterate thru all attributes
         for i,_type in enumerate(self.types):
             # get name and value of the attribute
             name = self.properties[i]
-            value = self.get_property(name)
+            value = self._properties[name]
             # for composite type, pack it looping over each subtype
             if type(_type) == type(()):
                 pack_func = getattr(self, 'pack_' + name, None)
